long SPEED = 1000L;
float RADIUS = 250f;
int NUM_WEDGES = 5;
int NUM_STATES = 30;
color BACKGROUND = color(255);
float[][][] angs;
float[][] currentAngs;
boolean animateChart = false;
long startTime = 0L;

void setup()
{
  size(600, 400);
  frame.setResizable(true);
  noStroke(); 
  smooth();
  frameRate(60);
  randomizeAngs();

  initCurrentAngles();

}

void initCurrentAngles()
{
  currentAngs = new float[NUM_WEDGES][2]; 

  for (int i = 0; i < NUM_WEDGES; i++)
  {
    currentAngs[i][0] = angs[0][i][0];
    currentAngs[i][1] = angs[0][i][1];
    //println("wedge " + i + " ang/rad = " + currentAngs[i][0] + "/" + currentAngs[i][1]);
  }
}

void randomizeAngs()
{
  angs = new float[NUM_STATES][NUM_WEDGES][2];

  for (int i = 0; i < NUM_STATES; i++)
  {
    float[][] state = new float[NUM_WEDGES][2];
    float totalAngle = 0f;
    float totalRadius = 0f;
    for (int j = 0; j < NUM_WEDGES; j++)
    {
      state[j][0] = random(.1f, 1f);
      state[j][1] = random(.75f, 1.25f);

      totalAngle += state[j][0];
      totalRadius += state[j][1];  
    }

    //normalize angles so that they all add up to 360f
    for (int j = 0; j < NUM_WEDGES; j++)
    {
      state[j][0] /= totalAngle;
    }
    angs[i] = state;
  }
}

void draw()
{
  background(BACKGROUND);
  if (animateChart == true)
  {
    updateChart();  
  } 

  float lastAng = 0;

  for (int i = 0; i < NUM_WEDGES; i++)
  {
    float arcAngle = currentAngs[i][0] * 360f;
    float arcRadius = currentAngs[i][1] * RADIUS; 
    fill(arcAngle * 3.0);
    arc(width/2, height/2, 
    arcRadius, arcRadius, 
    lastAng, lastAng+radians(arcAngle));
    lastAng += radians(arcAngle);  
  }
  
  println("fps: " + frameRate);
}


void updateChart()
{
  int prevState = (int) (( (millis() - startTime ) / SPEED) % NUM_STATES); 
  int nextState = (int) ((prevState + 1) % NUM_STATES);

  long turnMillis = (millis() - startTime) % SPEED;

  float perc = (float) turnMillis / (float) SPEED;

  //println("perc = " + perc);
  float[][] prevAngs = angs[prevState];                
  float[][] nextAngs = angs[nextState];                

  for (int i = 0; i < NUM_WEDGES; i++)
  {
    currentAngs[i][0] = prevAngs[i][0] + ((nextAngs[i][0] - prevAngs[i][0]) * perc);
    currentAngs[i][1] = prevAngs[i][1] + ((nextAngs[i][1] - prevAngs[i][1]) * perc);
  }
}

void keyPressed()
{  
  if (key == 'a')
  {
    startTime = millis();
    initCurrentAngles();
    animateChart = true;
  }
  if (key == 'b')
  {
    animateChart = false;
  }
}



